package LATMOS::Accounts::Web::Controller::Root;

use strict;
use warnings;
use base 'Catalyst::Controller';

#
# Sets the actions in this controller to be registered with no prefix
# so they function identically to actions created in MyApp.pm
#
__PACKAGE__->config->{namespace} = '';

=head1 NAME

LATMOS::Accounts::Web::Controller::Root - Root Controller for LATMOS::Accounts::Web

=head1 DESCRIPTION

[enter your description here]

=head1 METHODS

=cut

sub auth_required : Private {
    my ( $self, $c ) = @_;
    $c->res->status(401);
    $c->res->content_type('text/plain');
    $c->res->body('Authorization required.');
    $c->res->headers->push_header(
        'WWW-Authenticate' => 'Basic realm="Link::Accounts"'
    );
}

sub begin : Private {
    my ( $self, $c ) = @_;

    $c->log->debug($c->config->{ssl}->{HTTP_SSL_CLIENT_I_DN});
    if ($c->user_exists) {
        # Set login user:
        $c->model('Accounts')->db->{_user} = $c->user->{username};
    } else {
        # No need to login for About section
        if (!$c->authenticate({}, 'la')) {
            $c->forward('auth_required');
            return;
        }
    }
}

=head2 default

=cut

sub default : Private {
    my ( $self, $c ) = @_;
    $c->res->redirect($c->uri_for('/users', $c->user->{username})) if($c->user);
}

sub login : Local {
    my ( $self, $c ) = @_;

    $c->stash->{template} = 'login.tt';
    if ($c->req->param('username')) {
        if ($c->authenticate({
                username => $c->req->param('username'),
                password => $c->req->param('password')})) {
            my $redirurl = $c->uri_for($c->req->param('loguri') ||  '/');
            $c->res->redirect($redirurl || $c->uri_for('/'));
        } else {
            # invalid login...
        }
    }
    my ($redirurl) = grep { $_ } (
        $c->req->param('loguri'),
        ($c->req->path ? '/' . $c->req->path : undef),
        $c->uri_for('/'),
    );
    $c->stash->{redirurl} = $redirurl;
}

sub no_object : Local {
    my ($self, $c) = @_;
    $c->stash->{template} = 'no_object.tt';
}

sub obj_to_uri : Private {
    my ($self, $c, $ref) = @_;
    my $uri_part = {
        user => 'users',
        group => 'groups',
        nethost => 'nethosts',
        netzone => 'netzones',
        site => 'sites',
    }->{$ref} || $ref;
}

=head2 end

Attempt to render a view, if needed.

=cut

sub end : ActionClass('RenderView') {
    my ($self, $c) = @_;
    $c->forward($c->view('TT')) unless($c->res->body);
    $c->model('Accounts')->db->rollback;
    $c->model('Accounts')->call_batch_sync;
}

=head1 AUTHOR

Thauvin Olivier

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
