package LATMOS::Accounts::Web::Model::AttrFilter;

use strict;
use warnings;
use base 'Catalyst::Model';

=head1 NAME

LATMOS::Accounts::Web::Model::AttrFilter - Catalyst Model

=head1 DESCRIPTION

Catalyst Model.

=cut

my $fields = {
    user => [ qw(uid sn givenName description uidNumber gidNumber company department manager) ],
    group => [ qw(cn description gidNumber sutype) ],
    site => [ qw(l) ],
    nethost => [ qw(name cname otherName ip macaddr owner serialNumber) ],
    netzone => [ qw(net group type site) ],
    aliases => [ qw(as) ],
};

sub new {
    my ($class) = @_;
    return bless({}, $class);
}

sub ACCEPT_CONTEXT {
    my ($self, $c, $otype) = @_;

    my $new = {};
    $new->{c} = $c;
    $new->{otype} = $otype;
    $c->stash->{filter} = {
        otype => $otype,
    } unless($c->stash->{filter});
    $c->stash->{ofilter} = bless($new, __PACKAGE__);

    $c->stash->{db} ||= $c->model('Accounts')->db; # need in stash ?
    my $base = $c->stash->{db};

    my @objlist = ();
    if ($c->req->params->{'q'}) {
        my %objs;
        foreach my $attr (@{ $c->config->{objects}{$otype}{quick_search} || [ 'cn' ]}) {
            foreach ($base->search_objects(
                    $otype, "$attr~" . $c->req->params->{'q'})) {
                $objs{$_} = 1;
            }
            @objlist = sort keys %objs;
        }
    } elsif (exists($c->req->params->{'attr'})) {
        $c->stash->{filter}{'attr'} = [ $c->req->param('attr') ];
        $c->stash->{filter}{'attrval'} = [ $c->req->param('attrval') ];
        my @filter = $c->stash->{ofilter}->filter;
        @objlist = $base->search_objects($otype, @filter);
    } else {
        @objlist = $base->list_objects($otype);
    }

    $c->stash->{objectslist} = \@objlist;

    my $start = $c->req->param('start');
    $c->stash->{uparams} = { %{ $c->req->params || {} } };
    my %initials;
    my @sublist;
    foreach (@{ $c->stash->{objectslist} }) {
        $initials{substr($_, 0, 1)} = 1;
        if ($start) {
            if (index($_, $start) == 0) {
                push(@sublist, $_);
            }
        }
    }
    $c->stash->{sublist} = [ @sublist ];
    $c->stash->{initials} = [ sort keys %initials ];

    $c->stash->{ofilter}
}

sub otype {
    my ($self) = @_;
    $self->{otype};
}

sub submit {
    my ($self) = @_;
    return sprintf(
        '<input type="submit" name="%s">',
        $self->escape($self->label),
    );
}

sub current_value {
    my ($self) = @_;
    my @vals = @{ $self->{c}->stash->{filter}{attrval} || [] };
    my @fies = @{ $self->{c}->stash->{filter}{attr} || [] };
    my @couples;
    foreach (@fies) {
        my $val = shift(@vals);
        $_ && $val or next;
        push(@couples, { f => $_, v => $val});
    }
    push(@couples, {});
    @couples;
}

sub count_field {
    my ($self) = @_;
    return [ (0 .. (scalar($self->current_value) -1)) ];
}

sub search_field {
    my ($self, $count) = @_;
    my $form = $self->{c}->model('AttrForms', '', $self->otype, $self->{c}->stash->{db});
    my $html = '<select name="attr">
    <option value=""></option>';
    my @values = $self->current_value;
    foreach (@{ $fields->{$self->otype} }) {
        $html .= sprintf('<option value="%s"%s>%s</option>' . "\n",
            $_,
            (($values[$count]->{f} || '') eq $_ ? ' selected="on"' : ''),
            $form->attr_label($_),
        );
    }

    $html .= '</select>';
}

sub val_field {
    my ($self, $count) = @_;
    my @values = $self->current_value;
    return sprintf('<input type="text" name="attrval" value="%s">',
        $values[$count]->{v} || '');
}

sub filter {
    my ($self) = @_;
    map { $_->{f} . '~' .  $_->{v} } grep { $_->{f} } $self->current_value;
}

=head1 AUTHOR

Thauvin Olivier

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
